﻿using System;
using System.Collections.Generic;
using System.Linq;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "AutoSuggestBox 엘리먼트 : QuerySubmitted 이벤트 사용하기";

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 자동 제안 박스 텍스트 변경시 처리하기 - autoSuggestBox_TextChanged(autoSuggestBox, e)

        /// <summary>
        /// 자동 제안 박스 텍스트 변경시 처리하기
        /// </summary>
        /// <param name="autoSuggestBox">자동 제안 박스</param>
        /// <param name="e">이벤트 인자</param>
        private void autoSuggestBox_TextChanged(AutoSuggestBox autoSuggestBox, AutoSuggestBoxTextChangedEventArgs e)
        {
            if(e.Reason == AutoSuggestionBoxTextChangeReason.UserInput)
            {
                List<ControlInfoDataItem> resultList = GetControlInfoDataItemList(autoSuggestBox.Text);

                if(resultList.Count > 0)
                {
                    autoSuggestBox.ItemsSource = resultList;
                }
                else
                {
                    autoSuggestBox.ItemsSource = new string[] { "검색 결과가 없습니다." };
                }
            }
        }

        #endregion
        #region 자동 제안 박스 질의 제출시 처리하기 - autoSuggestBox_QuerySubmitted(autoSuggestBox, e)

        /// <summary>
        /// 자동 제안 박스 질의 제출시 처리하기
        /// </summary>
        /// <param name="autoSuggestBox">자동 제안 박스</param>
        /// <param name="e">이벤트 인자</param>
        private void autoSuggestBox_QuerySubmitted(AutoSuggestBox autoSuggestBox, AutoSuggestBoxQuerySubmittedEventArgs e)
        {
            if(e.ChosenSuggestion != null && e.ChosenSuggestion is ControlInfoDataItem)
            {
                SetControlData(e.ChosenSuggestion as ControlInfoDataItem);
            }
            else if(!string.IsNullOrEmpty(e.QueryText))
            {
                List<ControlInfoDataItem> resultList = GetControlInfoDataItemList(autoSuggestBox.Text);

                if(resultList.Count > 0)
                {
                    SetControlData(resultList.FirstOrDefault());
                }
            }
        }

        #endregion
        #region 자동 제안 박스 제안 선택시 처리하기 - autoSuggestBox_SuggestionChosen(autoSuggestBox, e)

        /// <summary>
        /// 자동 제안 박스 제안 선택시 처리하기
        /// </summary>
        /// <param name="autoSuggestBox">자동 제안 박스</param>
        /// <param name="e">이벤트 인자</param>
        private void autoSuggestBox_SuggestionChosen(AutoSuggestBox autoSuggestBox, AutoSuggestBoxSuggestionChosenEventArgs e)
        {
            if(e.SelectedItem is ControlInfoDataItem item)
            {
                autoSuggestBox.Text = item.Title;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 컨트롤 정보 데이터 항목 리스트 구하기 - GetControlInfoDataItemList(query)

        /// <summary>
        /// 컨트롤 정보 데이터 항목 리스트 구하기
        /// </summary>
        /// <param name="query">쿼리</param>
        /// <returns>컨트롤 정보 데이터 항목 리스트</returns>
        private List<ControlInfoDataItem> GetControlInfoDataItemList(string query)
        {
            List<ControlInfoDataItem> list = new List<ControlInfoDataItem>();

            string[] tokenArray = query.Split(" ");

            foreach(ControlInfoDataGroup group in ControlInfoDataSource.Instance.DataGroupList)
            {
                IEnumerable<ControlInfoDataItem> itemEnumerable = group.DataItemCollection.Where
                (
                    item =>
                    {
                        bool flag = true;

                        foreach(string token in tokenArray)
                        {
                            if(item.Title.IndexOf(token, StringComparison.CurrentCultureIgnoreCase) < 0)
                            {
                                flag = false;
                            }
                        }

                        return flag;
                    }
                );

                foreach(ControlInfoDataItem item in itemEnumerable)
                {
                    list.Add(item);
                }
            }

            return list.OrderByDescending
            (
                i => i.Title.StartsWith(query, StringComparison.CurrentCultureIgnoreCase)
            )
            .ThenBy(i => i.Title)
            .ToList();
        }

        #endregion
        #region 컨트롤 데이터 설정하기 - SetControlData(item)

        /// <summary>
        /// 컨트롤 데이터 설정하기
        /// </summary>
        /// <param name="item">컨트롤 정보 데이터 항목</param>
        private void SetControlData(ControlInfoDataItem item)
        {
            if(item != null)
            {
                this.relativePanel.Visibility = Visibility.Visible;

                BitmapImage bitmapImage = new BitmapImage(new Uri(item.ImagePath));

                this.image.Source = bitmapImage;

                this.titleTextBlock.Text    = item.Title;
                this.subtitleTextBlock.Text = item.Subtitle;
            }
        }

        #endregion
    }
}